@@ -15,6 +15,7 @@
 ## - as.list
 ## - as.data.frame
 ## - pseriesfy
+## - arrange for dplyr compatibility
 
 ## pseries:
 ## - [
@@ -26,6 +27,9 @@
 ## - print.summary
 ## - is.pseries
 
+## - pseries2pdataframe (non-exported)
+## - pmerge (non-exported)
+
 ## pdim:
 ## - pdim.default
 ## - pdim.data.frame
@@ -57,7 +61,7 @@ fancy.row.names <- function(index, sep = "-") {
 
 
 
-#' data.frame for panel data
+#' pdata.frame: a data.frame for panel data
 #' 
 #' An object of class 'pdata.frame' is a data.frame with an index
 #' attribute that describes its individual and time dimensions.
@@ -137,7 +141,7 @@ fancy.row.names <- function(index, sep = "-") {
 #'     as.data.frame),
 #' @param name the name of the `data.frame`,
 #' @param value the name of the variable to include,
-#' @param \dots further arguments.
+#' @param \dots further arguments passed on to internal usage of  `data.frame`.
 #' @return a `pdata.frame` object: this is a `data.frame` with an
 #'     `index` attribute which is a `data.frame` with two variables,
 #'     the individual and the time indexes, both being factors.  The
@@ -149,7 +153,7 @@ fancy.row.names <- function(index, sep = "-") {
 #'     'pdata.frame' (and other objects), [pdim()] to check the
 #'     dimensions of a 'pdata.frame' (and other objects), [pvar()] to
 #'     check for each variable if it varies cross-sectionally and over
-#'     time.  To check if the time periods are consecutive per
+#'     time. To check if the time periods are consecutive per
 #'     individual, see [is.pconsecutive()].
 #' @keywords classes
 #' @examples
@@ -177,18 +181,27 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
                         stringsAsFactors = FALSE,
                         replace.non.finite = FALSE,
                         drop.NA.series = FALSE, drop.const.series = FALSE,
-                        drop.unused.levels = FALSE) {
+                        drop.unused.levels = FALSE, ...) {
 
-    if (inherits(x, "pdata.frame")) stop("already a pdata.frame")
+    if(inherits(x, "pdata.frame")) {
+      if(!is.pdata.frame(x)) {
+        # check properties beyond class and if non-compliant, create proper
+        # pdata.frame by continuing with pdata.frame function
+        wrn.txt <- paste0("input 'x' claims to be a pdata.frame but does not have ",
+                          "compliant properties, so tried to re-create a compliant ",
+                          "pdata.frame from 'x'")
+        warning(wrn.txt)
+      }
+    }
   
-    if (length(index) > 3L){
+    if(length(index) > 3L){
         stop("'index' can be of length 3 at the most (one index variable for individual, time, group)")
     }
     
     # prune input: x is supposed to be a plain data.frame. Other classes building
     # on top of R's data frame can inject attributes etc. that confuse functions
     # in pkg plm.
-    x <- data.frame(x)
+    x <- data.frame(x, ...)
     
     # if requested: coerce character vectors to factors
     if (stringsAsFactors) {
@@ -252,7 +265,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     # grouping variable, this should be the third element of the index
     # vector or any "group" named element of this vector
     group.name <- NULL
-    if (! is.null(names(index)) || length(index == 3L)){
+    if (! is.null(names(index)) || length(index) == 3L){
         if (! is.null(names(index))){
             grouppos <- match("group", names(index))
             if (! is.na(grouppos)){
@@ -294,7 +307,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         time.name <- time
     }
     
-    # if index is numeric, this indicats a balanced panel with no. of
+    # if index is numeric, this indicates a balanced panel with no. of
     # individuals equal to id.name
     if(is.numeric(id.name)){
         if(!is.null(time.name))
@@ -319,7 +332,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
     }
     else{
         # id.name is not numeric, i.e., individual index is supplied
-        if (!id.name %in% names(x)) stop(paste("variable ", id.name, " does not exist (individual index)", sep=""))
+        if (!id.name %in% names(x)) stop(paste("variable '", id.name, "' does not exist (individual index)", sep=""))
         if (is.factor(x[[id.name]])){
             id <- x[[id.name]] <- x[[id.name]][drop = TRUE] # drops unused levels of factor
         }
@@ -333,10 +346,10 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
             # for the automatic addition of time index to be
             # successful if no time index was supplied
             x <- x[order(x[[id.name]]), ]
-            Ti <- table(x[[id.name]]) # was: Ti <- table(id)
+            Ti <- collapse::qtable(x[[id.name]])
             n <- length(Ti)
             time <- c()
-            for (i in 1:n){
+            for (i in seq_len(n)){
                 time <- c(time, 1:Ti[i])
             }
             time.name <- "time"
@@ -347,7 +360,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         else{
             # use supplied time index
             if (!time.name %in% names(x))
-                stop(paste0("variable ", time.name, " does not exist (time index)"))
+                stop(paste0("variable '", time.name, "' does not exist (time index)"))
             
             if (is.factor(x[[time.name]])){
                 time <- x[[time.name]] <- x[[time.name]][drop = TRUE]
@@ -391,9 +404,9 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
         x <- x[ , -posindex, drop = FALSE]
         if (ncol(x) == 0L) warning("after dropping of index variables, the pdata.frame contains 0 columns")
     }
-    
+
     ### warn if duplicate couples
-    test_doub <- table(index[[1L]], index[[2L]], useNA = "ifany")
+    test_doub <- collapse::qtable(index[[1L]], index[[2L]], na.exclude = FALSE) # == base R's table(x, y) # == table(index[[1L]], index[[2L]], useNA = "ifany")
     if (any(as.vector(test_doub[!is.na(rownames(test_doub)), !is.na(colnames(test_doub))]) > 1L))
       warning(paste("duplicate couples (id-time) in resulting pdata.frame\n to find out which,",
                     "use, e.g., table(index(your_pdataframe), useNA = \"ifany\")"))
@@ -487,7 +500,7 @@ pdata.frame <- function(x, index = NULL, drop.index = FALSE, row.names = TRUE,
 #   
 #   ## two sanity checks as [.pseries-subsetting was introduced in Q3/2021 and some packages
 #   ## produced illegal pseries (these pkg errors were fixed by new CRAN releases but maybe
-#   ## other code outhere produces illegal pseries, so leave these sanity checks in here for
+#   ## other code out there produces illegal pseries, so leave these sanity checks in here for
 #   ## a while, then remove (for speed)
 #     if(is.null(index)) warning("pseries object with is.null(index(pseries)) == TRUE encountered")
 #     if(!is.null(index) && !is.index(index)) warning(paste0("pseries object has illegal index with class(index) == ", paste0(class(index), collapse = ", ")))
@@ -594,7 +607,7 @@ subset_pseries <- function(x, ...) {
         # Kevin Tappe 2016-01-04 : in case of indexing (subsetting) a 
         # pdata.frame by a character, the subsetting vector should be 
         # converted to numeric by matching to the row names so that the 
-        # index can be correctly subsetted (by this numeric value).
+        # index can be correctly subset (by this numeric value).
         # Motivation:
         # Row names of the pdata.frame and row names of the pdata.frame's 
         # index are not guaranteed to be the same!
@@ -644,10 +657,10 @@ subset_pseries <- function(x, ...) {
     mydata <- eval(sc_mod)
 
     if (is.null(dim(mydata))) {
-      # if dim is NULL, subsetting did not return a data frame but  a vector or a
+      # if dim is NULL, subsetting did not return a data frame but a vector or a
       #   factor or NULL (nothing more is left)
       if (is.null(mydata)) {
-        # since R 3.4.0 NULL cannot have attributes, so special case it
+        # since R 3.4.0, NULL cannot have attributes, so special case it
         res <- NULL
       } else {
         # vector or factor -> make it a pseries
@@ -698,7 +711,7 @@ print.pdata.frame <- function(x, ...) {
   class(x) <- "data.frame"
   # This is a workaround: print.data.frame cannot handle
   # duplicated row names which are currently possible for pdata frames
-  if (anyDuplicated(rownames(x))) {
+  if (collapse::any_duplicated(rownames(x))) {
       print("Note: pdata.frame contains duplicated row names, thus original row names are not printed")
       rownames(x) <- NULL 
   }
@@ -837,6 +850,10 @@ as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, kee
       # do as base::as.data.frame does for NULL
       x <- as.data.frame(x, row.names = NULL)
     } else {
+      # argument check
+      if(!(isTRUE(row.names) || isFALSE(row.names) || is.character(row.names)))
+        stop("argument 'row.names' is none of NULL, FALSE, TRUE, and not a character")
+      
       if(is.logical(row.names) && row.names == FALSE) {
         # set row names to integer sequence 1, 2, 3, ...
         x <- as.data.frame(x)
@@ -851,8 +868,7 @@ as.data.frame.pdata.frame <- function(x, row.names = NULL, optional = FALSE, kee
         x <- as.data.frame(x)
         row.names(x) <- row.names
       }
-      if(!(isTRUE(row.names) || isFALSE(row.names) || is.character(row.names)))
-        stop("argument 'row.names' is none of NULL, FALSE, TRUE, and not a character")
+      
       # using row.names(x) <- "something" is safer (does not allow
       # duplicate row.names) than # attr(x,"row.names") <- "something"
     }
@@ -958,7 +974,7 @@ is.pseries <- function(object) {
 #'     observations (individual, time) actually used for model
 #'     estimation are taken into account.  When called on a
 #'     `(p)data.frame`, the rows in the `(p)data.frame` are
-#'     considered, disregarding any `NA`values in the dependent or
+#'     considered, disregarding any `NA` values in the dependent or
 #'     independent variable(s) which would be dropped during model
 #'     estimation.
 #' @export
@@ -1001,7 +1017,7 @@ pdim.default <- function(x, y, ...) {
   if (length(x) != length(y)) stop("The length of the two inputs differs\n")
   x <- x[drop = TRUE] # drop unused factor levels so that table() 
   y <- y[drop = TRUE] # gives only needed combinations
-  z <- table(x,y)
+  z <- collapse::qtable(x, y) ## == base R's table(x, y)
   Ti <- rowSums(z) # faster than: apply(z, 1, sum)
   nt <- colSums(z) #              apply(z, 2, sum)
   n <- nrow(z)
@@ -1011,8 +1027,8 @@ pdim.default <- function(x, y, ...) {
   id.names <- rownames(z)
   time.names <- colnames(z)
   panel.names <- list(id.names = id.names, time.names = time.names)
-  balanced <- if(any(as.vector(z) == 0)) FALSE else TRUE
-  if(any(as.vector(z) > 1)) stop("duplicate couples (id-time)\n")
+  balanced <- if(any(z <- as.vector(z) == 0)) FALSE else TRUE
+  if(any(z > 1)) stop("duplicate couples (id-time)\n")
   Tint <- list(Ti = Ti, nt = nt)
   z <- list(nT = nT, Tint = Tint, balanced = balanced, panel.names = panel.names)
   class(z) <- "pdim"
@@ -1094,6 +1110,143 @@ print.pdim <- function(x, ...) {
   invisible(pdim)
 }
 
+########### is.pbalanced ##############
+### for convenience and to be faster than pdim() for the purpose
+### of the determination of balancedness only, because it avoids
+### pdim()'s calculations which are unnecessary for balancedness.
+###
+### copied (and adapted) methods and code from pdim.*
+### (only relevant parts to determine balancedness)
+
+
+#' Check if data are balanced
+#' 
+#' This function checks if the data are balanced, i.e., if each individual has
+#' the same time periods
+#' 
+#' Balanced data are data for which each individual has the same time periods.
+#' The returned values of the `is.pbalanced(object)` methods are identical
+#' to `pdim(object)$balanced`.  `is.pbalanced` is provided as a short
+#' cut and is faster than `pdim(object)$balanced` because it avoids those
+#' computations performed by `pdim` which are unnecessary to determine the
+#' balancedness of the data.
+#' 
+#' @aliases is.pbalanced
+#' @param x an object of class `pdata.frame`, `data.frame`,
+#'     `pseries`, `panelmodel`, or `pgmm`,
+#' @param y (only in default method) the time index variable (2nd index
+#' variable),
+#' @param index only relevant for `data.frame` interface; if
+#'     `NULL`, the first two columns of the data.frame are
+#'     assumed to be the index variables; if not `NULL`, both
+#'     dimensions ('individual', 'time') need to be specified by
+#'     `index` as character of length 2 for data frames, for
+#'     further details see [pdata.frame()],
+#' @param \dots further arguments.
+#' @return A logical indicating whether the data associated with
+#'     object `x` are balanced (`TRUE`) or not
+#'     (`FALSE`).
+#' @seealso [punbalancedness()] for two measures of
+#'     unbalancedness, [make.pbalanced()] to make data
+#'     balanced; [is.pconsecutive()] to check if data are
+#'     consecutive; [make.pconsecutive()] to make data
+#'     consecutive (and, optionally, also balanced).\cr
+#'     [pdim()] to check the dimensions of a 'pdata.frame'
+#'     (and other objects), [pvar()] to check for individual
+#'     and time variation of a 'pdata.frame' (and other objects),
+#'     [pseries()], [data.frame()],
+#'     [pdata.frame()].
+#' @export
+#' @keywords attribute
+#' @examples
+#' 
+#' # take balanced data and make it unbalanced
+#' # by deletion of 2nd row (2nd time period for first individual)
+#' data("Grunfeld", package = "plm")
+#' Grunfeld_missing_period <- Grunfeld[-2, ]
+#' is.pbalanced(Grunfeld_missing_period)     # check if balanced: FALSE
+#' pdim(Grunfeld_missing_period)$balanced    # same
+#' 
+#' # pdata.frame interface
+#' pGrunfeld_missing_period <- pdata.frame(Grunfeld_missing_period)
+#' is.pbalanced(Grunfeld_missing_period)
+#' 
+#' # pseries interface
+#' is.pbalanced(pGrunfeld_missing_period$inv)
+#' 
+is.pbalanced <- function(x, ...) {
+  UseMethod("is.pbalanced")
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.default <- function(x, y, ...) {
+  if (length(x) != length(y)) stop("The length of the two inputs differs\n")
+  x <- x[drop = TRUE] # drop unused factor levels so that table 
+  y <- y[drop = TRUE] # gives only needed combinations
+  z <- collapse::qtable(x, y) # == base R's table(x, y)
+  balanced <- if(any(v <- as.vector(z) == 0L)) FALSE else TRUE
+  if (any(v > 1L)) warning("duplicate couples (id-time)\n")
+  balanced
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.data.frame <- function(x, index = NULL, ...) {
+  x <- pdata.frame(x, index)
+  index <- unclass(attr(x, "index")) # unclass for speed
+  is.pbalanced(index[[1L]], index[[2L]])
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pdata.frame <- function(x, ...) {
+  index <- unclass(attr(x, "index")) # unclass for speed
+  is.pbalanced(index[[1L]], index[[2L]])
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pseries <- function(x, ...) {
+  index <- unclass(attr(x, "index")) # unclass for speed
+  is.pbalanced(index[[1L]], index[[2L]])
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pggls <- function(x, ...) {
+  # pggls is also class panelmodel, but take advantage of its pdim attribute
+  attr(x, "pdim")$balanced
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pcce <- function(x, ...) {
+  # pcce is also class panelmodel, but take advantage of its pdim attribute
+  attr(x, "pdim")$balanced
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pmg <- function(x, ...) {
+  # pmg is also class panelmodel, but take advantage of its pdim attribute
+  attr(x, "pdim")$balanced
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.pgmm <- function(x, ...) {
+  # pgmm is also class panelmodel, but take advantage of its pdim attribute
+  attr(x, "pdim")$balanced
+}
+
+#' @rdname is.pbalanced
+#' @export
+is.pbalanced.panelmodel <- function(x, ...) {
+  x <- model.frame(x)
+  is.pbalanced(x)
+}
+
 #' Extract the indexes of panel data
 #' 
 #' This function extracts the information about the structure of the
@@ -1217,7 +1370,10 @@ index.panelmodel <- function(x, which = NULL, ...) {
 is.index <- function(index) {
   # not exported, helper function
   # checks if the index is an index in the sense of package plm
-  if(all(class(index) == c("pindex", "data.frame"))) TRUE else FALSE
+  res <- TRUE
+  if(!all(class(index) == c("pindex", "data.frame"))) res <- FALSE
+  if(!is.null(n <- ncol(index)) && !(n %in% c(2, 3))) res <- FALSE
+  res
 }
 
 has.index <- function(object) {
@@ -1243,7 +1399,7 @@ checkNA.index <- function(index, which = "all", error = TRUE) {
   if(which == "all") {
     if(anyNA(index[[1L]])) feedback("NA in the individual index variable")
     if(anyNA(index[[2L]])) feedback("NA in the time index variable")
-    n.index <- if(inherits(index, "pindex")) ncol(index) else length(index) # else-branche is list (for speed)
+    n.index <- if(inherits(index, "pindex")) ncol(index) else length(index) # else-branch is list (for speed)
     if(n.index == 3L) { if(anyNA(index[[3L]])) feedback("NA in the group index variable") }
   }
   if(which == 1L) {
@@ -1255,8 +1411,29 @@ checkNA.index <- function(index, which = "all", error = TRUE) {
   if(which == 3L) {
     if(anyNA(index[[3L]])) feedback("NA in the group index variable")
   }
+  NULL
 }
 
+
+make.fdindex <- function(x) {
+  ## non-exported helper function
+  ## constructs an index suitable for time-wise first-difference data
+  ## input: an index (class c("pindex", "data.frame"))
+  ## return value: plain 2-entry list of the index factors
+  ix <- unclass(x)
+  
+  ix.ind.lag <- lag(add_pseries_features(ix[[1L]], ix))
+  ix.ti.lag  <- lag(add_pseries_features(ix[[2L]], ix))
+
+  na <- is.na(ix.ind.lag) # NAs are in same positions for ind and time index
+
+  ix.ind.lag <- ix.ind.lag[!na]
+  ix.ti.lag  <- ix.ti.lag[!na]
+  
+  list(ix.ind.lag, ix.ti.lag)
+}
+
+
 # pos.index:
 # not exported, helper function
 #
@@ -1275,4 +1452,117 @@ pos.index <- function(x, ...) {
   index_pos <- match(index_names, names(x))
   names(index_pos) <- index_names
   return(index_pos)
-}
\ No newline at end of file
+}
+
+is.pdata.frame <- function(x, feedback = NULL) {
+  # not exported, helper function
+  # checks if a pdata.frame has appropriate properties
+
+  res <- TRUE
+  if(!inherits(x, "pdata.frame")) res <- FALSE
+  if(!has.index(x))               res <- FALSE
+  if(!nrow(index(x)) == nrow(x))  res <- FALSE
+  
+  if(!res && !is.null(feedback)) {
+    feedback <- switch(feedback,
+                       "error" = stop,
+                       "warn"  = warning)
+    feedback.txt <- paste0("input data claims to be a pdata.frame but does not seem to have compliant properties, ",
+                           "results can be unreliable. This can happen due to data manipulation by ",
+                           "non-pdata.frame-aware functions (e.g., 'dplyr' on pdata.frame). \n Maybe re-create ",
+                           "data input as fresh pdata.frame after last data manipulation with other tools.")
+    feedback(feedback.txt)
+  }
+  res
+}
+
+check.pdata.frame <- function(x) {
+  # not exported, helper function
+  # checks if a pdata.frame has appropriate properties
+  # Like is.pdata.frame but with error output about ill-specification
+  stopifnot(inherits(x, "pdata.frame"))
+  stopifnot(has.index(x))
+  stopifnot(nrow(index(x)) == nrow(x))
+  NULL
+}
+
+pseries2pdataframe <- function(x, pdata.frame = TRUE, ...) {
+  ## non-exported
+  ## Transforms a pseries in a (p)data.frame with the indices as regular columns
+  ## in positions 1, 2 and (if present) 3 (individual index, time index, group index).
+  ## if pdataframe = TRUE -> return a pdata.frame, if FALSE -> return a data.frame
+  ## ellipsis (dots) passed on to pdata.frame()
+  if(!inherits(x, "pseries")) stop("input needs to be of class 'pseries'")
+  indices <- attr(x, "index")
+  class(indices) <- setdiff(class(indices), "pindex")
+  vx <- remove_pseries_features(x)
+  dfx <- cbind(indices, vx)
+  dimnames(dfx)[[2L]] <- c(names(indices), deparse(substitute(x)))
+  res <- if(pdata.frame == TRUE) {
+    pdata.frame(dfx, index = names(indices), ...)
+  } else { dfx }
+  return(res)
+}
+
+pmerge <- function(x, y, ...) {
+  ## non-exported
+  ## Returns a data.frame, not a pdata.frame.
+  ## pmerge is used to merge pseries or pdata.frames into a data.frame or
+  ## to merge a pseries to a data.frame
+  
+  ## transf. if pseries or pdata.frame
+  if(inherits(x, "pseries")) x <- pseries2pdataframe(x, pdata.frame = FALSE)
+  if(inherits(y, "pseries")) y <- pseries2pdataframe(y, pdata.frame = FALSE)
+  if(inherits(x, "pdata.frame")) x <- as.data.frame(x, keep.attributes = FALSE)
+  if(inherits(y, "pdata.frame")) y <- as.data.frame(y, keep.attributes = FALSE)
+  
+  # input to merge() needs to be data.frames; not yet suitable for 3rd index (group variable)
+  z <- merge(x, y,
+             by.x = dimnames(x)[[2L]][1:2],
+             by.y = dimnames(y)[[2L]][1:2], ...)
+  return(z)
+}
+
+## dplyr "compatibility"/awareness/warning, see https://github.com/ycroissant/plm/issues/46
+# test in test_pdata.frame_compliant.R
+# 
+# Approach: Avoid dplyr-dependency albeit registering methods for it, needs the
+# utils::globalVariables() statement to avoid a NOTE in R CMD check about 
+# undefined global variable.
+# Shall CRAN ever increase the code checking strictness, the approach with a
+# Suggests-dependency is given commented below as well.
+utils::globalVariables("arrange")
+#' @rawNamespace if(getRversion() >= "3.6.0") {S3method(dplyr::arrange, pdata.frame)}
+arrange.pdata.frame <- function(.data, ..., .by_group = FALSE, .locale = NULL) {
+  # function signature of dplyr:::arrange.data.frame
+  idx <- index(.data)
+  ag.data <- NextMethod()
+  ag.idx <- arrange(idx, ..., .by_group = FALSE, .locale = NULL) # dispatches to arrange.pindex
+  attr(ag.data, "index") <- ag.idx
+  ag.data
+}
+
+#' @rawNamespace if(getRversion() >= "3.6.0") {S3method(dplyr::arrange, pindex)}
+arrange.pindex <- function(.data, ..., .by_group = .by_group, .locale = .locale) {
+  NextMethod()
+}
+
+### alternatives
+# arrange.pdata.frame <- function(.data, ..., .by_group = FALSE, .locale = NULL) {
+#   ### this alternative would require dplyr to be at least a Suggests-dependency
+#   stopifnot(requireNamespace("dplyr"))
+#   idx <- index(.data)
+#   ag.df  <- dplyr::arrange(.data, ..., .by_group = FALSE, .locale = NULL)
+#   ag.idx <- dplyr::arrange(idx,   ..., .by_group = FALSE, .locale = NULL)
+#   attr(ag.df, "index") <- ag.idx
+#   ag.df
+# }
+
+# arrange.pdata.frame <- function(.data, ..., .by_group = FALSE, .locale = NULL) {
+#   # this alternative just warns and executes dplyr's plain arrange destroying the index
+#   wrn <- paste0("dplyr::arrange not safe on pdata.frames!\n",
+#                 "Before input to estimation and test functions of package plm, ",
+#                 "create a fresh pdata.frame from the dplyr-manipulated data to be safe")
+#   warning(wrn)
+#   NextMethod()
+# }
