@@ -191,26 +191,29 @@ fixef.plm <- function(object, effect = NULL,
                        "time"       = pdim$Tint$nt)
       
       s2 <- deviance(object) / df.residual(object)
-      if (type != "dfirst") {
-        sefixef <- sqrt(s2 / nother + apply(Xb[, nw, drop = FALSE], 1,
-                                            function(x) t(x) %*% vcov %*% x))
-      } else {
-        Xb <- t(t(Xb[-1, ]) - Xb[1L, ])
-        sefixef <- sqrt(s2 * (1 / nother[-1] + 1 / nother[1])+
-                          apply(Xb[, nw, drop = FALSE], 1,
-                                function(x) t(x) %*% vcov %*% x))
-      }
+      
+      sefixef <- if (type != "dfirst") {
+                    sqrt(s2 / nother + apply(Xb[, nw, drop = FALSE], 1,
+                                            function(x) t(x) %*% vcov %*% x) )
+                  } else {
+                    Xb <- t(t(Xb[-1L, , drop = FALSE]) - Xb[1L, ])
+                    sqrt(s2 * (1 / nother[-1L] + 1 / nother[1L]) +
+                                      apply(Xb[, nw, drop = FALSE], 1,
+                                            function(x) t(x) %*% vcov %*% x) )
+                  }
+
       res <- switch(type,
                       "level"  = fixef,
-                      "dfirst" = fixef[2:length(fixef)] - fixef[1L],
+                      "dfirst" = fixef[seq_along(fixef)[-1L]] - fixef[1L],
                       "dmean"  = (fixef - weighted.mean(fixef, w = nother)))
       
       res <- structure(res, se = sefixef, class = c("fixef", "numeric"),
-                type = type, df.residual = df.residual(object))  
+                       type = type, df.residual = df.residual(object))  
     } else {
     ## case model.effect == "twoways"
     ##  * two-way balanced/unbalanced model for all effects
-
+    ## TODO: SEs are not computed in this case yet
+    ##       (can be computed for effect = "individual" and "time"; also for "twoways"?)
       beta.data <- as.numeric(tcrossprod(coef(object),
                                          model.matrix(object, model = "pooling")[ , nw, drop = FALSE]))
       yhat <- object$model[ , 1L] - object$residuals
@@ -239,8 +242,7 @@ fixef.plm <- function(object, effect = NULL,
         ## other dfirst
         other.fixef.dfirst <- other.fixef.lvl - other.fixef.lvl[1L]
         tw.fixef.lvl <- tw.fixef.lvl - other.fixef.dfirst[indexl[[other.idx]]]
-        
-        tw.fixef.lvl <- tw.fixef.lvl[!duplicated(indexl[[idx]])]
+        tw.fixef.lvl <- tw.fixef.lvl[!collapse::fduplicated(indexl[[idx]], all = FALSE)]
         names(tw.fixef.lvl) <- pdim[["panel.names"]][[idx]]
       } else {
         # effect = "twoways": everything already computed, just set names
@@ -250,7 +252,7 @@ fixef.plm <- function(object, effect = NULL,
       
       res <- switch(type,
                       "level"  = tw.fixef.lvl,
-                      "dfirst" = tw.fixef.lvl[2:length(tw.fixef.lvl)] - tw.fixef.lvl[1L],
+                      "dfirst" = tw.fixef.lvl[seq_along(tw.fixef.lvl)[-1L]] - tw.fixef.lvl[1L],
                       "dmean"  = {
                           if(pdim$balanced || effect == "twoways") {
                             tw.fixef.lvl - mean(tw.fixef.lvl)
@@ -425,8 +427,8 @@ ranef.plm <- function(object, effect = NULL, ...) {
     # in the unbalanced cases, ercomp[["theta"]] is full length (# obs)
     #  -> reduce to per id/time
     select <- switch(effect,
-                     "individual" = !duplicated(index(object$model)[1L]),
-                     "time"       = !duplicated(index(object$model)[2L]))
+                     "individual" = !collapse::fduplicated(index(object$model)[1L], all = FALSE),
+                     "time"       = !collapse::fduplicated(index(object$model)[2L], all = FALSE))
     theta <- theta[select]
   }
   
@@ -462,7 +464,7 @@ ranef.plm <- function(object, effect = NULL, ...) {
 #' computationally more demanding than just taking the weighted
 #' mean. However, with `within_intercept` one also gets the
 #' associated standard error and it is possible to get an overall
-#' intercept for twoway fixed effect models.
+#' intercept for two-way fixed effect models.
 #' 
 #' Users can set argument `vcov` to a function to calculate a
 #' specific (robust) variance--covariance matrix and get the
@@ -532,8 +534,8 @@ ranef.plm <- function(object, effect = NULL, ...) {
 #' ## have a model returned
 #' mod_fe_int <- within_intercept(gi, return.model = TRUE)
 #' summary(mod_fe_int)
-#' # replicates Stata's robust standard errors
-#' summary(mod_fe_int, vcvov = function(x) vcovHC(x, type = "sss")) 
+#' # replicates Stata's robust standard errors exactly as model is with intercept
+#' summary(mod_fe_int, vcov = function(x) vcovHC(x, type = "sss")) 
 # 
 within_intercept <- function(object, ...) {
   UseMethod("within_intercept")
@@ -546,7 +548,7 @@ within_intercept <- function(object, ...) {
 #       system interprets '.intercept' as a class called 'intercept'.
 
 # Note: return value of within_intercept is related to return values of fixef.plm,
-#       see tests/test_within_intercept.R
+#       see inst/tests/test_within_intercept.R
 
 #' @rdname within_intercept
 #' @export
